<?php
/*
Documentation: https://github.com/square/connect-api-examples/tree/master/connect-examples/v2/php_payment
               https://docs.connect.squareup.com/payments/online-payments
*/
if( !class_exists( 'MUSIC_STORE_SQUARE_ADDON' ) )
{
    class MUSIC_STORE_SQUARE_ADDON
    {
		private static $instance;
		private $licenses;
		private $default_settings;
		private $current_settings;

		public static function init()
		{
			if(self::$instance == null)
			{
				self::$instance = new self();
			}

			return self::$instance;
		} // End init

		public function __construct()
		{
			$this->licenses = [];
			$this->default_settings = array(
				'enabled' => 0,
				'square_key'  => '',
				'square_accesstoken' => '',
				'mode' => 1,
				'label' => 'Square'
			);

			if(!is_admin())
			{
				add_action('musicstore_calling_payment_gateway', array($this, 'payment_gateway_integration'), 10, 2);
				add_action('musicstore_checking_payment', array($this, 'check_payment'), 1);
				add_filter('musicstore_payment_gateway_enabled', array($this, 'is_enabled'), 10);
				add_filter('musicstore_payment_gateway_list', array($this, 'populate_list'), 10);
				add_action('wp_footer', array($this, 'square_form'), 10);
			}
			else
			{
				add_action('musicstore_settings_page', array(&$this, 'show_settings'), 11);
				add_action('musicstore_save_settings', array(&$this, 'save_settings'), 11);
			}
		} // End __construct

		/**************************** PRIVATE METHODS ***************************/

		private function _settings($reload = false)
		{
			if(empty($this->current_settings) || $reload)
				$this->current_settings = get_option('ms_square_settings', $this->default_settings);

			return $this->current_settings;
		} // End _settings

		private function _set_license($license_type)
		{
			if(function_exists('music_store_set_license'))
			{
				music_store_set_license( $license_type, $this->licenses );
			}
		} // End _set_license

		/**************************************	PUBLIC METHODS	**************************************/

		public function is_enabled($enabled)
		{
			$settings = $this->_settings();
			return $settings['enabled'] || $enabled;
		} // End is_enabled

		public function populate_list($payment_gateways)
		{
			if($this->is_enabled(false))
			{
				$settings = $this->_settings();
				$label = (!empty($settings['label'])) ? __($settings['label'], MS_TEXT_DOMAIN) : $this->default_settings['label'];
				$payment_gateways['square'] = $label;
			}
			return $payment_gateways;
		} // End populate_list

		public function show_settings()
		{
			$settings = $this->_settings(true);
			?>
			<div id="metabox_basic_settings" class="postbox" >
				<h3 class='hndle' style="padding:5px;"><span>Square <?php _e('Payment Gateway', MS_TEXT_DOMAIN); ?></span></h3>
				<div class="inside">
					<table class="form-table">
						<tr>
							<th scope="row"><?php _e('Enable Square?', MS_TEXT_DOMAIN); ?></th>
							<td>
								<input type="checkbox" name="square_enabled" <?php print (!empty($settings['enabled']) ? 'checked' : ''); ?> />
							</td>
						</tr>
						<tr>
							<th scope="row"><?php _e('Label', MS_TEXT_DOMAIN); ?></th>
							<td><input type="text" name="square_label" style="width:100%;" value="<?php print esc_attr(!empty($settings['label']) ? $settings['label'] : ''); ?>" /></td>
						</tr>
						<tr>
							<th scope="row">Squareup.com <?php _e('Application ID', MS_TEXT_DOMAIN); ?></th>
							<td>
								<input type="text" name="square_key" style="width:100%;" value="<?php echo esc_attr($settings["square_key"]); ?>" />
							</td>
						</tr>
						<tr>
							<th scope="row">Squareup.com <?php _e('Access Token', MS_TEXT_DOMAIN); ?></th>
							<td>
								<input type="text" name="square_accesstoken" style="width:100%;" value="<?php echo esc_attr($settings["square_accesstoken"]); ?>" />
							</td>
						</tr>
						<tr valign="top">
							<th scope="row"><?php _e('Square Mode', MS_TEXT_DOMAIN); ?></th>
							<td>
								<select name="square_mode">
									<option value="1" <?php if ($settings["mode"] != '0') echo 'selected'; ?>>
										<?php print esc_html(__('Production - real payments processed', MS_TEXT_DOMAIN)); ?>
									</option>
									<option value="0" <?php if ($settings["mode"] == '0') echo 'selected'; ?>>
										<?php print esc_html(__('SandBox - Square testing sandbox area', MS_TEXT_DOMAIN)); ?>
									</option>
								</select>
							</td>
						</tr>
                   </table>
				</div>
			</div>
            <?php
		} // End show_settings

		public function save_settings()
		{
			$settings = array(
				'enabled' => isset($_REQUEST['square_enabled']) ? 1 : 0,
				'square_key'  => sanitize_text_field($_REQUEST['square_key']),
				'square_accesstoken' => sanitize_text_field($_REQUEST['square_accesstoken']),
				'mode' => @intval($_REQUEST['square_mode']),
				'label' => sanitize_text_field($_REQUEST['square_label'])
			);
			$this->current_settings = $settings;
			update_option('ms_square_settings', $settings);
		} // End save_settings

        /************************ PUBLIC METHODS  *****************************/

		/**
         * Inserts banks and Check if the Optional is enabled in the form, and inserts radiobutton
         */
        public function	square_form()
		{
			global $music_store_settings;

            if($this->is_enabled(false))
			{
				$settings = $this->_settings();
				if(!empty($settings['square_key']) && !empty($settings['square_accesstoken']))
				{
					?>
					<link rel="stylesheet" type="text/css" href="<?php print plugins_url('/square-addon/css/sq-payment-form.css', __FILE__ ); ?>">
					<script type="text/javascript" src="https://js.<?php print $settings['mode'] ? 'squareup' : 'squareupsandbox'; ?>.com/v2/paymentform"></script>
					<script type="text/javascript" src="<?php print plugins_url('/square-addon/js/ms-square.js', __FILE__ ); ?>"></script>
					<script>
						var ms_square_form_template = ''+
						'<div class="sq-payment-form" style="visibility:hidden;">'+
						'	<div class="sq-close-form"><a href="javascript:void(0);">X</a></div>'+
						'	<div class="sq-field">'+
						'	  	<label class="sq-label"><?php print esc_js(__('Email', MS_TEXT_DOMAIN)); ?></label>'+
						'	  	<div id="sq-email"><input type="email" id="ms_email" name="ms_email" class="sq-input" placeholder="<?php esc_attr_e(__('Required email', MS_TEXT_DOMAIN)); ?>" /></div>'+
						'	</div>'+
						'	<div id="sq-walletbox">'+
						'  		<button id="sq-google-pay" class="button-google-pay"></button>'+
						'  		<button id="sq-apple-pay" class="sq-apple-pay"></button>'+
						'  		<button id="sq-masterpass" class="sq-masterpass"></button>'+
						'  		<div class="sq-wallet-divider">'+
						'			<span class="sq-wallet-divider__text"><?php print esc_js(__('Or', MS_TEXT_DOMAIN)); ?></span>'+
						'  		</div>'+
						'	</div>'+
						'	<div id="sq-ccbox">'+
						'		<div class="sq-field">'+
						'	  		<label class="sq-label"><?php print esc_js(__('Card Number', MS_TEXT_DOMAIN)); ?></label>'+
						'	  		<div id="sq-card-number"></div>'+
						'		</div>'+
						'		<div class="sq-field-wrapper">'+
						'	  		<div class="sq-field sq-field--in-wrapper">'+
						'				<label class="sq-label">CVV</label>'+
						'				<div id="sq-cvv"></div>'+
						'	  		</div>'+
						'	  		<div class="sq-field sq-field--in-wrapper">'+
						'				<label class="sq-label"><?php print esc_js(__('Expiration', MS_TEXT_DOMAIN)); ?></label>'+
						'				<div id="sq-expiration-date"></div>'+
						'	  		</div>'+
						'		</div>'+
						'		<button id="sq-creditcard" class="sq-button" onclick="msOnGetCardNonce(event)"></button>'+
						'		<div id="error"></div>'+
						'		<input type="hidden" id="card-nonce" name="sqnonce">'+
						'	</div>'+
						'</div>';

						window.applicationId = "<?php print esc_js($settings['square_key']); ?>";
						jQuery(window).on('load', function(){
							var $ = jQuery,
								ms_buy_now_square = ('ms_buy_now' in window) ? ms_buy_now : function(){ return true; };
							window['ms_buy_now'] = function(e, p){
								var o = $(e),
									f = o.closest('form'),
									l = f.find('select');
								ms_form = f;
								if(!l.length || l.val() == 'square')
								{
                                    if(ms_buy_now_square(e, p))
                                    {
                                        if(typeof ms_loading_square_form != 'undefined')
                                        {
                                            return false;
                                        }
                                        else
                                        {
                                            $('.sq-payment-form').remove();
                                            ms_loading_square_form = 1;
                                        }
                                        $.ajax({
                                            dataType: 'json',
                                            method: 'POST',
                                            url: f.attr('action'),
                                            data: f.serialize(),
                                            headers: {'X-Alt-Referer': document.referrer},
                                            success: function(data)
                                            {
                                                if(
                                                    typeof data == 'object' &&
                                                    'name' in data &&
                                                    'amount' in data
                                                )
                                                {
                                                    var currency = '<?php echo esc_js(strtolower((!empty($music_store_settings['ms_paypal_currency'])) ? $music_store_settings['ms_paypal_currency'] : 'usd')); ?>',
                                                        amount = parseFloat(data['amount']);

                                                    $('body').append(ms_square_form_template).find('#sq-creditcard').html((amount/100).toFixed(2)+currency);
                                                    ms_square_form_generator(
                                                        data['name'],
                                                        parseFloat(data['amount']),
                                                        currency,
                                                        data['notifyurl'],
                                                        data['returnurl']
                                                    );
                                                }
                                            }
                                        });
                                    }
									return false;
								}
                                else
                                {
                                    return ms_buy_now_square(e, p);
                                }
							};
							jQuery(document).on('click', '.ms-purchase-button', function(){return ms_buy_now(this);});
						});
					</script>
					<?php
				}
			}
		} // End square_form


        public function payment_gateway_integration($amount, $purchase_settings)
		{
			global $music_store_settings;

			if(
				!$this->is_enabled(false) ||
				!isset($_REQUEST['ms_payment_gateway']) ||
				$_REQUEST['ms_payment_gateway'] != 'square'
			) return;

			if($amount > 0)
			{
				$item_name = (!empty($purchase_settings['item_name'])) ? $purchase_settings['item_name'] : '';
				$item_name = html_entity_decode($item_name, ENT_COMPAT, 'UTF-8');
				if(function_exists('music_store_apply_taxes')) $amount = music_store_apply_taxes($amount);
				$amount = round($amount*100,0);
				$settings = $this->_settings();
				$currency = strtolower((!empty($music_store_settings['ms_paypal_currency'])) ? $music_store_settings['ms_paypal_currency'] : 'usd');

				$purchase_id = (!empty($purchase_settings['id'])) ? $purchase_settings['id'] : '';
				$baseurl 	= $purchase_settings['baseurl'];
				$returnurl 	= $purchase_settings['returnurl'].'&purchase_id='.$purchase_id;
				$cancelurl 	= $purchase_settings['cancelurl'];

				if(!empty($purchase_settings['products']))
				{
					foreach($purchase_settings['products'] as $product)
					{
						$products_id .= (isset($product->product_id)) ? $product->product_id : $product->ID;
					}
				}
				$coupon_code = (!empty($purchase_settings['coupon'])) ? '|ms_coupon_code='.$purchase_settings['coupon'] : '';
				$buyer = (($user_id = get_current_user_id()) != 0) ? '|buyer_id='.$user_id : '';

				$output = array(
					'amount'	=> $amount,
					'baseurl' 	=> $baseurl,
					'returnurl' => $returnurl,
					'name'		=> $item_name,
					'notifyurl' => apply_filters('musicstore_notify_url', $baseurl.'|pid='.$products_id.'|purchase_id='.$purchase_id.'|rtn_act=purchased_product_music_store'.$coupon_code.$buyer)
				);

				// If there is not the token, returns a json with teh amount information
				if(!headers_sent())
				{
					header('Content-Type: application/json');
				}
				print json_encode($output);
				exit;
			}
		} // End payment_gateway_integration

		public function check_payment()
		{
			global $music_store_settings, $wpdb;

			if(
				$this->is_enabled(false) &&
				isset($_REQUEST['ms_square_ipncheck'])
			)
			{
				$settings = $this->_settings();
				if(!empty($settings['square_key']) && !empty($settings['square_accesstoken']))
				{
					try
					{
						try
						{
							$_parameters = explode('|', $_GET['ms-action']);
							foreach($_parameters as $_parameter)
							{
								$_parameter_parts = explode('=',$_parameter);
								if(count($_parameter_parts) == 2)
								{
									$ipn_parameters[$_parameter_parts[0]] = sanitize_text_field($_parameter_parts[1]);
								}
							}

							$purchase_id = isset($ipn_parameters['purchase_id']) ? $ipn_parameters['purchase_id'] : 0;
							$product_id = isset($ipn_parameters['pid']) ? $ipn_parameters['pid'] : 0;
							// If the buyer_id is empty the result would be 0
							$GLOBALS['buyer_id'] = @intval($ipn_parameters['buyer_id']);

							$products = array();
							if(!empty($music_store_settings['ms_paypal_shopping_cart']))
							{
								if(!empty($purchase_id))
									$products = music_store_getProducts($purchase_id);
							}
							elseif(!empty($product_id))
							{
								$_post = get_post($product_id);
								if(is_null($_post))
								{
									_e('Non-existent product', MS_TEXT_DOMAIN);
									exit;
								}

								switch ($_post->post_type)
								{
									case "ms_song":
										$products[] = new MSSong($_post->ID);
									break;
									case "ms_collection":
										$products[] = new MSCollection($_post->ID);
									break;
									default:
										_e('The product is not a song or collection', MS_TEXT_DOMAIN);
										exit;
									break;
								}
							}

							$percent = 0;
							$discount_note = '';
							$base_price = 0;
							$payer_email = (!empty($_REQUEST['payer_email'])) ? sanitize_text_field($_REQUEST['payer_email']) : '';
							$item_name = (!empty($_REQUEST['item_name'])) ? sanitize_text_field($_REQUEST['item_name']) : '';

							// Walking the products list to get the determine the price applied
							foreach($products as $i => $product)
							{
								$processed_product = new stdClass;
								$processed_product->product_id = (isset($product->product_id)) ? $product->product_id : $product->id;

								if(isset($product->price_type) && $product->price_type == 'exclusive' && !empty($product->exclusive_price))
								{
									$processed_product->exclusive_price_applied = true;
									$processed_product->final_price = $product->exclusive_price;
									$this->_set_license('exclusive');
								}
								else
								{
									if(function_exists('music_store_getValidProductDiscount'))
											$discount = music_store_getValidProductDiscount($product->product_id);

									$processed_product->exclusive_price_applied = false;
									$processed_product->final_price = (!empty($discount)) ? $discount->discount : $product->price;
									$processed_product->discount_applied = (!empty($discount) && !empty($discount->note)) ? ' - '.$discount->note : '';
									$this->_set_license( 'regular');
								}
								$products[$i] = $processed_product;
								$base_price += $processed_product->final_price;
							}

							// Coupon
							if(($coupon_code = (!empty($ipn_parameters['coupon'])) ? $ipn_parameters['coupon'] : '') != '')
							{
								$coupon = $wpdb->get_row(
									$wpdb->prepare("SELECT * FROM ".$wpdb->prefix.MSDB_COUPON." WHERE coupon=%s", $coupon_code)
								);

								if(!empty($coupon))
								{
									$percent = $coupon->discount;
									$discount_note = ' - Coupon code: '.$coupon->coupon.' discount: '.$percent.'%';
								}
							}

							// Checks the store discount
							if(function_exists('music_store_getValidStoreDiscount'))
							{
								$store_discount = music_store_getValidStoreDiscount($base_price);
								if($store_discount && $percent < $store_discount->discount){
									$percent = $store_discount->discount;
									$discount_note = ' - '.$store_discount->note;
									$coupon_code = '';
								}
							}

							$amount = $base_price*(100-$percent);
							if(function_exists('music_store_apply_taxes')) $amount = music_store_apply_taxes($amount);

							if (!class_exists('\Square\SquareClient'))
							require_once dirname( __FILE__ ) . '/square-addon/vendor/autoload.php';

							$access_token = $settings['square_accesstoken'];

                            $api_client = new \Square\SquareClient([
                                'accessToken' => $access_token,
                                'environment' => ($settings['mode'] ? \Square\Environment::PRODUCTION : \Square\Environment::SANDBOX)
                            ]);


							$nonce = $_REQUEST['ms_square_ipncheck'];
							$payments_api = $api_client->getPaymentsApi();

                            $money = new \Square\Models\Money();
                            $money->setAmount($amount);
                            $money->setCurrency($music_store_settings['ms_paypal_currency']);

                            $create_payment_request = new \Square\Models\CreatePaymentRequest($nonce, uniqid(), $money);

                            try
                            {
                                $result = $payments_api->createPayment($create_payment_request);
                                if ($result->isError())
                                {
                                    echo 'Api response has Errors';
                                    echo $result->getErrors();
                                    exit();
                                }
                            } catch (ApiException $e) {
                                echo 'Caught exception!<br>';
                                echo $e->getMessage();
                                exit();
                            }

							$payment_gateway_data = json_encode($result);

							// Register the purchase
							foreach($products as $product)
							{
								$note = strip_tags(((!empty($product->discount_applied)) ? $product->discount_applied : '').$discount_note);

								if(
									function_exists('music_store_register_purchase') &&
									music_store_register_purchase(
										$product->product_id,
										$purchase_id,
										(!empty($payer_email)) ? $payer_email : $result->payment->id,
										round($product->final_price*(100-$percent)/100,2),
										$payment_gateway_data,
										$note
									)
								)
								{
									if($product->exclusive_price_applied)
									{
										$wpdb->query(
											$wpdb->prepare("UPDATE ".$wpdb->prefix.MSDB_POST_DATA." SET purchases=purchases+1, purchased_exclusively=1 WHERE id=%d", $product->product_id)
										);
										$wpdb->query(
											$wpdb->prepare("UPDATE ".$wpdb->posts." SET post_status='pexclusively' WHERE ID=%d", $product->product_id)
										);
									}
									else
									{
										$wpdb->query(
											$wpdb->prepare("UPDATE ".$wpdb->prefix.MSDB_POST_DATA." SET purchases=purchases+1 WHERE id=%d", $product->product_id)
										);
									}
								}
							}
							if(function_exists('music_store_removeCart')) music_store_removeCart($purchase_id);
							if(!empty($payer_email) && function_exists('music_store_send_emails'))
							{
								music_store_send_emails(
									array(
										'item_name' 	=> $item_name,
										'currency'		=> $music_store_settings['ms_paypal_currency'],
										'purchase_id'	=> $purchase_id,
										'amount'		=> number_format($amount/100, 2),
										'payer_email'	=> $payer_email
									),
									$this->licenses
								);
							}

							$_POST['ms_purchase_id'] = $purchase_id;
							$_POST['ms_payment_amount'] = number_format($amount/100, 2);

							do_action('musicstore_payment_received', $_POST, $products);

							echo 'ok';
							exit;

						}
						catch (\SquareConnect\ApiException $e)
						{
                                echo '<p>Square: '.__('Failed to process credit card. Error message', MS_TEXT_DOMAIN).': <strong>'. $e->getResponseBody()->errors[0]->detail.'</strong></p>';
                                echo '<p>'.__('Please <a href="javascript:window.history.back();">go back and try again</a>', MS_TEXT_DOMAIN).'</p>';
                                exit;
						}
					}
					catch (Exception $e)
					{
						echo '<p>Square: '.__('Failed to process credit card. Error message', MS_TEXT_DOMAIN).': <strong>'. $e->getMessage().'</strong></p>';
						echo '<p>'.__('Please <a href="javascript:window.history.back();">go back and try again</a>', MS_TEXT_DOMAIN).'</p>';
						exit;
					}
				}
			}
		} // end check_payment
    } // End MUSIC_STORE_SQUARE_ADDON
}
MUSIC_STORE_SQUARE_ADDON::init();