<?php
if( !defined( 'MS_H_URL' ) ) { echo 'Direct access not allowed.';  exit; }

// ************************************** WIDGETS CLASSES *********************************************/

/**
 * MSProductListWidget Class
 */
class MSProductListWidget extends WP_Widget {

    /** constructor */
    function __construct() {
        parent::__construct(false, $name = 'Music Store Products List');

    }

    function widget($args, $instance) {
        extract( $args );
        $title = apply_filters('widget_title', $instance['title']);

        $defaults = array( 'list_type' => 'top_rated', 'columns' => 1, 'number' => 3 );
		$instance_p = wp_parse_args( (array) $instance, $defaults );

        $list_type  = $instance_p[ 'list_type' ];
        $columns    = $instance_p[ 'columns' ];
        $number     = $instance_p[ 'number' ];

        $atts = array( 'type' => $list_type, 'columns' => $columns, 'number' => $number );
        echo $before_widget;
		if ( $title ) echo $before_title . $title . $after_title;
		echo $GLOBALS['music_store']->load_product_list($atts);
		echo $after_widget;
    }

    function update($new_instance, $old_instance) {
        $instance = $old_instance;

		/* Strip tags (if needed) and update the widget settings. */
		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['columns'] = max( @intval($new_instance['columns']), 1 );
		$instance['number'] = max( @intval($new_instance['number']), 1 );
		$instance['list_type'] = $new_instance['list_type'];

		return $instance;
    }

    function form( $instance ) {

        /* Set up some default widget settings. */
		$defaults = array( 'title' => '', 'list_type' => 'top_rated', 'columns' => 1, 'number' => 3 );
		$instance = wp_parse_args( (array) $instance, $defaults );

        $title      = $instance[ 'title' ];
        $list_type  = $instance[ 'list_type' ];
        $columns    = $instance[ 'columns' ];
        $number     = $instance[ 'number' ];

        ?>
            <p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label></p>

            <p>
                <label for="<?php echo $this->get_field_id('list_type'); ?>"><?php _e('Select the type of list:', MS_TEXT_DOMAIN); ?><br />
                    <select style="width:100%;" id="<?php echo $this->get_field_id( 'list_type' ); ?>" name="<?php echo $this->get_field_name( 'list_type' ); ?>">
                        <option value="top_rated" <?php if( $list_type == 'top_rated' ) echo 'SELECTED'; ?> >Top Rated</option>
                        <option value="new_products" <?php if( $list_type == 'new_products' ) echo 'SELECTED'; ?> >New Products</option>
                        <option value="top_selling" <?php if( $list_type == 'top_selling' ) echo 'SELECTED'; ?> >Top Selling</option>
                    </select>
                </label>
            </p>
            <p>
                <label for="<?php echo $this->get_field_id('number'); ?>"><?php _e('Enter the number of products to show:', MS_TEXT_DOMAIN); ?><br />
                    <input style="width:100%;" id="<?php esc_attr_e($this->get_field_id( 'number' )); ?>" name="<?php esc_attr_e($this->get_field_name( 'number' )); ?>" value="<?php esc_attr_e($number); ?>" />
                </label>
            </p>
            <p>
                <label for="<?php echo $this->get_field_id('columns'); ?>"><?php _e('Enter the number of columns:', MS_TEXT_DOMAIN); ?><br />
                    <input style="width:100%;" id="<?php esc_attr_e($this->get_field_id( 'columns' )); ?>" name="<?php esc_attr_e($this->get_field_name( 'columns' )); ?>" value="<?php esc_attr_e($columns); ?>" />
                </label>
            </p>
        <?php
    }

} // clase MSProductListWidget

/**
 * MSProductWidget Class
 */
class MSProductWidget extends WP_Widget {

    /** constructor */
    function __construct() {
        parent::__construct(false, $name = 'Music Store Product');
    }

    function widget($args, $instance) {
        extract( $args );
        $title = apply_filters('widget_title', $instance['title']);

        $defaults = array( 'product_id' => '' );
		$instance_p = wp_parse_args( (array) $instance, $defaults );

        $product_id  = $instance_p[ 'product_id' ];

        $atts = array( 'id' => $product_id );
        echo $before_widget;
		if ( $title ) echo $before_title . $title . $after_title;
		echo $GLOBALS['music_store']->load_product($atts);
		echo $after_widget;
    }

    function update($new_instance, $old_instance) {
        $instance = $old_instance;

		/* Strip tags (if needed) and update the widget settings. */
		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['product_id'] = $new_instance['product_id']*1;

		return $instance;
    }

    function form( $instance ) {
        /* Set up some default widget settings. */
		$defaults = array( 'title' => '', 'product_id' => '' );
		$instance = wp_parse_args( (array) $instance, $defaults );
        $title       = $instance[ 'title' ];
        $product_id  = $instance[ 'product_id' ];
        ?>
            <p><label for="<?php esc_attr_e($this->get_field_id('title')); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php esc_attr_e($this->get_field_id('title')); ?>" name="<?php esc_attr_e($this->get_field_name('title')); ?>" type="text" value="<?php esc_attr_e($title); ?>" /></label></p>

            <p>
                <label for="<?php esc_attr_e($this->get_field_id('product_id')); ?>"><?php _e('Enter the product ID:', MS_TEXT_DOMAIN); ?><br />
                    <input class="widefat" id="<?php esc_attr_e($this->get_field_id( 'product_id' )); ?>" name="<?php esc_attr_e($this->get_field_name( 'product_id' )); ?>" value="<?php esc_attr_e($product_id); ?>" />
                </label>
            </p>
        <?php
    }

} // clase MSProductWidget


/**
 * MSSalesCounterWidget Class
 */
class MSSalesCounterWidget extends WP_Widget {

    /** constructor */
    function __construct() {
        parent::__construct(false, $name = 'Music Store Sales Counter');

    }

    function widget($args, $instance) {
        extract( $args );
        $title = apply_filters('widget_title', $instance['title']);

        $style      = $instance[ 'style' ];
        $min_length = ( isset($instance[ 'min_length' ] ) ) ? max( @intval($instance[ 'min_length' ]), 1 ) : 1;

        $atts = array( 'style' => $style, 'min_length' => $min_length );
		echo $before_widget;
		if ( $title ) echo $before_title . $title . $after_title;
		echo $GLOBALS['music_store']->sales_counter( $atts );
        echo $after_widget;
    }

    function update($new_instance, $old_instance) {
        $instance = $old_instance;
		/* Strip tags (if needed) and update the widget settings. */
		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['min_length'] = max( @intval($new_instance['min_length']), 1 );
		$instance['style'] = $new_instance['style'];

		return $instance;
    }

    function form( $instance ) {

        /* Set up some default widget settings. */
		$defaults = array( 'title' => '', 'min_length' => 3 );

        // create an array to hold directory list
        // create a handler for the directory
        $handler = opendir( MS_CORE_IMAGES_PATH.'/counter' );
        $digit_design = '';
        // open directory and walk through the filenames

        while ($file = readdir($handler)) {

            // if file isn't this directory or its parent, add it to the results
            if ($file != "." && $file != "..") {
                if( is_dir( MS_CORE_IMAGES_PATH.'/counter/'.$file ) ){
                    if( !isset( $defaults[ 'style' ] )) $defaults[ 'style' ] = $file;
                    $digit_design .= '<option value="'.esc_attr($file).'" '.( ( isset( $instance[ 'style' ] ) && $instance[ 'style' ] == $file ) ? 'SELECTED' : '' ).' >'.music_store_strip_tags($file,true).'</option>';
                }
            }
        }

        // tidy up: close the handler
        closedir($handler);

        $instance   = wp_parse_args( (array) $instance, $defaults );
        $title      = $instance[ 'title' ];
        $min_length = $instance[ 'min_length' ];

		?>
        <p><label for="<?php esc_attr_e($this->get_field_id('title')); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php esc_attr_e($this->get_field_id('title')); ?>" name="<?php esc_attr_e($this->get_field_name('title')); ?>" type="text" value="<?php esc_attr_e($title); ?>" /></label></p>

        <p>
            <label for="<?php esc_attr_e($this->get_field_id('style')); ?>"><?php _e('Select the numbers design:', MS_TEXT_DOMAIN); ?><br />
                <select style="width:100%;" id="<?php esc_attr_e($this->get_field_id( 'style' )); ?>" name="<?php esc_attr_e($this->get_field_name( 'style' )); ?>">
                    <?php echo $digit_design; ?>
                </select>
            </label>
        </p>
        <p>
            <label for="<?php esc_attr_e($this->get_field_id('min_length')); ?>"><?php _e('Enter minimum length of counter:', MS_TEXT_DOMAIN); ?><br />
                <input style="width:100%;" id="<?php esc_attr_e($this->get_field_id( 'min_length' )); ?>" name="<?php esc_attr_e($this->get_field_name( 'min_length' )); ?>" value="<?php esc_attr_e($min_length); ?>" />
            </label>
        </p>
        <?php
    }

} // clase MSSalesCounterWidget

/**
 * MSLoginFormWidget Class
 */
class MSLoginFormWidget extends WP_Widget {

    /** constructor */
    function __construct() {
        parent::__construct(false, $name = 'Music Store Login Form');

    }

    function widget($args, $instance) {
        $user_obj = wp_get_current_user();

        extract( $args );
		echo $before_widget;
        $title = apply_filters('widget_title', $instance['title']);
        $store_url = $instance[ 'store_url' ];
        if ( $title ) echo $before_title . $title . $after_title;

        if($user_obj->ID)
        {
            echo '<p>'.$user_obj->display_name.'<br/><br/>
            <a href="'.esc_attr(wp_logout_url($store_url)).'">'.__('Logout', MS_TEXT_DOMAIN).'</a></p>';

        }
        else
        {
            wp_login_form(['redirect'=>$store_url]);
        }

        echo $after_widget;
    }

    function update($new_instance, $old_instance) {
        $instance = $old_instance;

		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['store_url'] = sanitize_text_field($new_instance['store_url']);

		return $instance;
    }

    function form( $instance ) {
        global $music_store_settings;

        /* Set up some default widget settings. */
		$defaults = array( 'title' => '', 'store_url' => !empty($music_store_settings) && !empty($music_store_settings['ms_main_page']) ? $music_store_settings['ms_main_page'] : '' );

        $instance   = wp_parse_args( (array) $instance, $defaults );
        $title      = $instance[ 'title' ];
        $store_url  = $instance[ 'store_url' ];

		?>
        <p><label for="<?php esc_attr_e($this->get_field_id('title')); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php esc_attr_e($this->get_field_id('title')); ?>" name="<?php esc_attr_e($this->get_field_name('title')); ?>" type="text" value="<?php esc_attr_e($title); ?>" /></label></p>
        <p>
            <label for="<?php esc_attr_e($this->get_field_id('store_url')); ?>"><?php _e('Enter the URL to the store page:', MS_TEXT_DOMAIN); ?><br />
                <input class="widefat" id="<?php esc_attr_e($this->get_field_id( 'store_url' )); ?>" name="<?php esc_attr_e($this->get_field_name( 'store_url' )); ?>" type="text" value="<?php esc_attr_e($store_url); ?>" />
            </label>
        </p>
        <?php
    }

} // clase MSLoginFormWidget